==> mls-0.1.tar.gz: mls-0.1/src/panel_select.cpp <==
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <vector>
#include <stack>


#include "panel.h"

using namespace std;
using namespace MLS;


// 내부static 변수를 리턴함 쓰레드에 대해서 안전하지 않음
// 가져온 스트링의 마지막이 / 로 끝나면 디렉토리 입니다.
// zip 파일일때는 bDirExpansion은 true가 되선 않된다.
vector<string> &Panel::GetSelection(int &size, bool bDirExpansion)
{
	static vector<string> sel;
	stack<string, vector<string> > dirs;
	sel.clear();
	size=0;
	
	if (sel_num == 0)
	{
		if (file_list[cur]->name != "..")
		{
			if (m_nZip == 0)
			{
				if (bDirExpansion && file_list[cur]->bDir)
					dirs.push(file_list[cur]->name + '/');
				else
				{
					sel.push_back(file_list[cur]->name);
					size += file_list[cur]->size;
				}
			}
			else
			{
				// zip 이면 FullName을 넣는다. 
				sel.push_back(file_list[cur]->FullName);
				size += file_list[cur]->size;
			}
		}
	}
	else	
	{
		for (pFileIterator i= file_list.begin(); i!=file_list.end(); ++i)
		{
			if ((*i)->selected)
			{
				if (m_nZip == 0)
				{
					if (bDirExpansion && (*i)->bDir)
						dirs.push((*i)->name + '/');
					else 
					{
						sel.push_back((*i)->name);
						size += (*i)->size;
					}
				}
				else
				{
					// zip 이면 FullName을 넣는다. 
					sel.push_back((*i)->FullName);
					if (!(*i)->bDir)
					{
						size += (*i)->size;
					}
				}
			}
		}
	}
	
	if (bDirExpansion) // Dir 밑으로 확장시킨다
	{
		DIR        *handle;
		dirent     *pFile;
		struct stat stat_bufs;		
		
		while(!dirs.empty())
		{
			string p = dirs.top();
			dirs.pop();
			
			chdir (path.c_str());
			if (chdir (p.c_str())==-1) continue;
			if ((handle = opendir("."))==NULL) continue;
			
			sel.push_back(p);
			
			while((pFile = readdir(handle)))
			{
				if (!strcmp(pFile->d_name, ".")) continue;
				if (!strcmp(pFile->d_name, "..")) continue;
				
				if (lstat(pFile->d_name, &stat_bufs)==-1) continue;
				if (S_ISDIR (stat_bufs.st_mode)) dirs.push(p + pFile->d_name + '/');
				else 
				{
					sel.push_back(pFile->d_name);
				
					size += stat_bufs.st_size;
				}
			}
			closedir(handle);			
		}
		chdir (path.c_str());
	}
	
	return sel;
}

void Panel::select(File &f)
{
	if (f.name == "..") return;	
	if (f.selected) return;
	// 인덱스를 저장
	f.selected = true;	
	sel_size += f.size;	
	sel_num++;
}

void Panel::deselect(File &f)
{
	if (!f.selected) return;
	f.selected = false;
	sel_size -= f.size;	
	sel_num--;		
}

==> mls-0.2.tar.gz: mls-0.2/src/panel_select.cpp <==
///	\file	panel_select.cpp
///	\brief	panel_select.cpp
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <vector>
#include <stack>


#include "panel.h"

using namespace std;
using namespace MLS;


///	\brief	선택된 파일 목록을 가져오는 함수
///
/// 내부static 변수를 리턴함 쓰레드에 대해서 안전하지 않음
/// 가져온 스트링의 마지막이 / 로 끝나면 디렉토리 입니다.
/// zip 파일일때는 bDirExpansion은 true가 되선 않된다.
///
///	\param	size			파일 목록 개수(반환값)
///	\param	bDirExpansion	Directory 확장여부
///	\return	선택된 파일 목록
vector<string> &Panel::GetSelection(int &size, bool bDirExpansion)
{
	static vector<string> sel;
	stack<string, vector<string> > dirs;
	sel.clear();
	size=0;
	
	if (sel_num == 0)
	{
		if (file_list[cur]->name != "..")
		{
			if (m_nZip == 0)
			{
				if (bDirExpansion && file_list[cur]->bDir)
					dirs.push(file_list[cur]->name + '/');
				else
				{
					sel.push_back(file_list[cur]->name);
					size += file_list[cur]->size;
				}
			}
			else
			{
				// zip 이면 FullName을 넣는다. 
				sel.push_back(file_list[cur]->FullName);
				size += file_list[cur]->size;
			}
		}
	}
	else	
	{
		for (pFileIterator i= file_list.begin(); i!=file_list.end(); ++i)
		{
			if ((*i)->selected)
			{
				if (m_nZip == 0)
				{
					if (bDirExpansion && (*i)->bDir)
						dirs.push((*i)->name + '/');
					else 
					{
						sel.push_back((*i)->name);
						size += (*i)->size;
					}
				}
				else
				{
					// zip 이면 FullName을 넣는다. 
					sel.push_back((*i)->FullName);
					if (!(*i)->bDir)
					{
						size += (*i)->size;
					}
				}
			}
		}
	}
	
	if (bDirExpansion) // Dir 밑으로 확장시킨다
	{
		DIR        *handle;
		dirent     *pFile;
		struct stat stat_bufs;		
		
		while(!dirs.empty())
		{
			string p = dirs.top();
			dirs.pop();
			
			chdir (path.c_str());
			if (chdir (p.c_str())==-1) continue;
			if ((handle = opendir("."))==NULL) continue;
			
			sel.push_back(p);
			
			while((pFile = readdir(handle)))
			{
				if (!strcmp(pFile->d_name, ".")) continue;
				if (!strcmp(pFile->d_name, "..")) continue;
				
				if (lstat(pFile->d_name, &stat_bufs)==-1) continue;
				if (S_ISDIR (stat_bufs.st_mode)) dirs.push(p + pFile->d_name + '/');
				else 
				{
					sel.push_back(pFile->d_name);
				
					size += stat_bufs.st_size;
				}
			}
			closedir(handle);			
		}
		chdir (path.c_str());
	}
	
	return sel;
}

///	\brief	파일을 선택한다.(반전 시킴)
///	\param	f	선택할 파일
void Panel::select(File &f)
{
	if (f.name == "..") return;	
	if (f.selected) return;
	// 인덱스를 저장
	f.selected = true;	
	sel_size += f.size;	
	sel_num++;
}

///	\brief	파일선택을 해제한다.
///	\param	f	선택 해제할 파일
void Panel::deselect(File &f)
{
	if (!f.selected) return;
	f.selected = false;
	sel_size -= f.size;	
	sel_num--;		
}
